/******************************************************************************
;  *       @型号                 : MC51F7424
;  *       @创建日期             : 2023.02.09
;  *       @公司/作者            : SINOMCU.COM-AE
;  *       @晟矽微官网           : http://www.sinomcu.com/
;  *       @版权                 : 晟矽微电 版权所有.
;  *----------------------摘要描述---------------------------------
;  * void ADC_Pin_Init(void) ADC引脚初始化;
;  * void ADC_Init(void) ADC初始化;
;  * void ADC_ISR(void) interrupt ADC_IRQn ADC中断服务程序.            
******************************************************************************/

#include "user_config.h"
#include "MC51F7424_COMMON.h"
#include "GPIO.h"
#include "ADC.h"
#include "ADC_user.h"

uint16_t xdata ADC_Result_Buff[ADC_CH_NUM] _at_ ADC_RESULT_XDATA_ADD;
flagstatus g_ADCScanTrigger =RESET;

/**
  * @brief  ADC pin initialization
  * @param  
  * @retval 
  * @note
  *     P26(AN4)
  */ 
void ADC_Pin_Init(void) {
  P26_HIGH_Z_MODE; /*AN4*/  
}

/**
  * @brief  initializaiton ADC for user application;
  *         a) Vref,Gain user_config.h;48MHz/3/(12+8);12Bit
  *         b) Auto scan 3channal:AN4,Intenal LDO,TmpSensor
  *         c) Enable ADCIF interrupt
  *         d) Calibration.
  * @param  
  * @retval 
  */ 
void ADC_Init(void) {
  ADC_Pin_Init();

  ADC_Disable_VrefPin();               /*set to default*/
  ADC_VrefPin_Set(ADC_VrefPin_PIN);    /*set to default*/
#if(VREF_mV==VDD_USER)
  /*Vref VDD*/
  ADC_Vref_Set(ADC_VREF_VDD);
  /*Gain=1*/
  ADC_Gain_Set(ADC_GAIN_1);
#else
  ADC_Vref_Set(ADC_VREF_LDO1P65);
  /*Gain=0.5*/
  ADC_Gain_Set(ADC_GAIN_0P5);
#endif
  ADC_Channel_Set(ADC_Channel_AN4);
  /*HIRC48M,div 3,sample 8 clk */
  ADC_Clk_HIRC48M(); 
  ADC_ClkDiv_Set(ADC_CLK_DIV_3);
  ADC_SampleClk_Set(ADC_SPS_CLK_8);
  /*result cover before,12bit*/
  ADC_Result_Cover(); 
  ADC_ResultDiv_Set(ADC_Result_DIV_1); /*set to default*/
  ADC_ResultACC_Set(ADC_ResultACC_1);  /*set to default*/
  ADC_Mode_Set(ADC_MODE_12BIT);

  /*internal temperature senser enable*/
  ADC_Enable_Tmp();
  /*auto scan 3 channel,set resualt xdata address*/ 
  Set_ADCAutoScan_ChNum(ADC_AutoScan_3CH);
  Set_ADCAutoScan_Xdata_Add(ADC_RESULT_XDATA_ADD);
  Set_ADCAutoScan_Xdata_Num(ADC_CH_NUM);
  Set_ADCAutoScan_CH1(ADC_Channel_AN4);
  Set_ADCAutoScan_CH2(ADC_Channel_LDO1P5);
  Set_ADCAutoScan_CH3(ADC_Channel_Tmp);
  ADC_Trig_OneAutoScan();
  ADC_Enable_AutoScan(); 

  ADC_Enable();

  /*start calibration and wait finish*/
  Start_ADC_Adjust(); 
  while(Get_ADC_AdjustState() ==ADC_ONGOING) {;}
  /*enable ADC interrupt*/
  ADC_IE_Enable();
  
  Start_ADC_Convert();
  while(g_ADCScanTrigger != SET) { ; }      /*AN22021 V1.1.0 2.5.1*/
  g_ADCScanTrigger = RESET;
  Start_ADC_Convert();
  while(g_ADCScanTrigger != SET) { ; }      /*AN22021 V1.1.0 2.5.1*/
  g_ADCScanTrigger = RESET;
}

/**
  * @brief  ADC interrupt function
  * @param  
  * @retval 
  */ 
void ADC_ISR(void) interrupt ADC_IRQn {
  if(Get_ADC_IF()) {
    Clear_ADC_IF();
    g_ADCScanTrigger = SET;
    Set_ADCAutoScan_Xdata_Add(ADC_RESULT_XDATA_ADD);  /*AN22021 V1.1.0 2.5.3*/
    Set_ADCAutoScan_Xdata_Num(ADC_CH_NUM);
  }
  if(Get_ADC_CMPLIF()) {
    Clear_ADC_CMPLIF();
  }
  if(Get_ADC_CMPGIF()) {
    Clear_ADC_CMPGIF();
  }
}

/******************************** END OF FILE *********************************/